==> PyTango-7.1.2.tar.gz: PyTango-7.1.2/src/to_py_numpy.hpp <==

#pragma once

/// @name Array extraction
/// @{

template <long tangoArrayTypeConst>
inline boost::python::object to_py_numpy(const typename TANGO_const2type(tangoArrayTypeConst)* tg_array, boost::python::object parent)
{
    typedef typename TANGO_const2type(tangoArrayTypeConst) TangoArrayType;

    static const int typenum = TANGO_const2scalarnumpy(tangoArrayTypeConst);

    if (tg_array == 0) {
        // Empty
        PyObject* value = PyArray_SimpleNew(0, 0, typenum);
        if (!value)
            boost::python::throw_error_already_set();
        return boost::python::object(boost::python::handle<>(value));
    }
    
    // Create a new numpy.ndarray() object. It uses ch_ptr as the data,
    // so no costy memory copies when handling big images.
    const void *ch_ptr = reinterpret_cast<const void *>(tg_array->get_buffer());
    int nd = 1;
    npy_intp dims[1];
    dims[0]= tg_array->length();
    PyObject* py_array = PyArray_SimpleNewFromData(nd, dims, typenum, const_cast<void*>(ch_ptr));
    if (!py_array) {
        boost::python::throw_error_already_set();
    }

    // numpy.ndarray() does not own it's memory, so we need to manage it.
    // We can assign a 'parent' object that will be informed (decref'd)
    // when the last copy of numpy.ndarray() disappears. That should
    // actually destroy the memory in its destructor
    PyObject* guard = parent.ptr();
    Py_INCREF(guard);
    PyArray_BASE(py_array) = guard;
    
    return boost::python::object(boost::python::handle<>(py_array));
}

template <>
inline boost::python::object to_py_numpy<Tango::DEVVAR_STRINGARRAY>(const Tango::DevVarStringArray* tg_array, boost::python::object parent)
{
    return to_py_list(tg_array);
}

template <>
inline boost::python::object to_py_numpy<Tango::DEVVAR_LONGSTRINGARRAY>(const Tango::DevVarLongStringArray* tg_array, boost::python::object parent)
{
    boost::python::list result;
    
    result.append(to_py_numpy<Tango::DEVVAR_LONGARRAY>(&tg_array->lvalue, parent));
    result.append(to_py_numpy<Tango::DEVVAR_STRINGARRAY>(&tg_array->svalue, parent));
    
    return result;
}

template <>
inline boost::python::object to_py_numpy<Tango::DEVVAR_DOUBLESTRINGARRAY>(const Tango::DevVarDoubleStringArray* tg_array, boost::python::object parent)
{
    boost::python::list result;
    
    result.append(to_py_numpy<Tango::DEVVAR_DOUBLEARRAY>(&tg_array->dvalue, parent));
    result.append(to_py_numpy<Tango::DEVVAR_STRINGARRAY>(&tg_array->svalue, parent));
    
    return result;
}
/// @}
// ~Array Extraction
// -----------------------------------------------------------------------

==> PyTango-7.1.3.tar.gz: PyTango-7.1.3/src/to_py_numpy.hpp <==
/*******************************************************************************

   This file is part of PyTango, a python binding for Tango

   http://www.tango-controls.org/static/PyTango/latest/doc/html/index.html

   (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
  
   This is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
  
   This software is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.
  
   You should have received a copy of the GNU Lesser General Public License
   along with this program; if not, see <http://www.gnu.org/licenses/>.
   
*******************************************************************************/

#pragma once

/// @name Array extraction
/// @{

template <long tangoArrayTypeConst>
inline boost::python::object to_py_numpy(const typename TANGO_const2type(tangoArrayTypeConst)* tg_array, boost::python::object parent)
{
    typedef typename TANGO_const2type(tangoArrayTypeConst) TangoArrayType;

    static const int typenum = TANGO_const2scalarnumpy(tangoArrayTypeConst);

    if (tg_array == 0) {
        // Empty
        PyObject* value = PyArray_SimpleNew(0, 0, typenum);
        if (!value)
            boost::python::throw_error_already_set();
        return boost::python::object(boost::python::handle<>(value));
    }
    
    // Create a new numpy.ndarray() object. It uses ch_ptr as the data,
    // so no costy memory copies when handling big images.
    const void *ch_ptr = reinterpret_cast<const void *>(tg_array->get_buffer());
    int nd = 1;
    npy_intp dims[1];
    dims[0]= tg_array->length();
    PyObject* py_array = PyArray_SimpleNewFromData(nd, dims, typenum, const_cast<void*>(ch_ptr));
    if (!py_array) {
        boost::python::throw_error_already_set();
    }

    // numpy.ndarray() does not own it's memory, so we need to manage it.
    // We can assign a 'parent' object that will be informed (decref'd)
    // when the last copy of numpy.ndarray() disappears. That should
    // actually destroy the memory in its destructor
    PyObject* guard = parent.ptr();
    Py_INCREF(guard);
    PyArray_BASE(py_array) = guard;
    
    return boost::python::object(boost::python::handle<>(py_array));
}

template <>
inline boost::python::object to_py_numpy<Tango::DEVVAR_STRINGARRAY>(const Tango::DevVarStringArray* tg_array, boost::python::object parent)
{
    return to_py_list(tg_array);
}

template <>
inline boost::python::object to_py_numpy<Tango::DEVVAR_LONGSTRINGARRAY>(const Tango::DevVarLongStringArray* tg_array, boost::python::object parent)
{
    boost::python::list result;
    
    result.append(to_py_numpy<Tango::DEVVAR_LONGARRAY>(&tg_array->lvalue, parent));
    result.append(to_py_numpy<Tango::DEVVAR_STRINGARRAY>(&tg_array->svalue, parent));
    
    return result;
}

template <>
inline boost::python::object to_py_numpy<Tango::DEVVAR_DOUBLESTRINGARRAY>(const Tango::DevVarDoubleStringArray* tg_array, boost::python::object parent)
{
    boost::python::list result;
    
    result.append(to_py_numpy<Tango::DEVVAR_DOUBLEARRAY>(&tg_array->dvalue, parent));
    result.append(to_py_numpy<Tango::DEVVAR_STRINGARRAY>(&tg_array->svalue, parent));
    
    return result;
}
/// @}
// ~Array Extraction
// -----------------------------------------------------------------------
